/**************************************************************************
 FILE:
	 osi_netctl.h

 DESCRIPTION:
 	Network layer routing and configuration ioctls and associated
	structures.
	
 @(#)osi_netctl.h 1.4 91/01/11 Copyr 1986 Sun Micro

 **************************************************************************/

/* ------------------------------------------------------------------------- */

/* Configuration information */

#define IND_SET_ROUTING_SOCK	_IOW(o,31,int)

/* Should the ES/IS stuff be per interface? */

/* New configuration information */
typedef struct ind_config {
	/* Modify address handling in the kernel to always use USER_DEFINED: */
	long		ic_user_defined_only;
	/* CLNP */
	long		ic_clnp_chksum;		/* CLNP checksums */
	long		ic_lifetime;		/* "-l" to osi.routed */
	long		ic_forwarding;		/* Turn on/off forwarding */
	/* ES/IS */
	long		ic_holding_time;
	long		ic_generate_chksum;	/* ESH,ISH,RD checksums */
	long		ic_multiple_nsap_esh;	/* 1 to turn on */
	int		ic_send_esct;		/* ESCT to send in ISH. 
						 * -1 if none */
} IND_CONFIG;

#define IND_SET_CONFIG		_IOW(o,34,struct ind_config)
#define IND_GET_CONFIG		_IOR(o,35,struct ind_config)

/* ------------------------------------------------------------------------- */

/* ES/IS configuration */

struct ind_if_info {
	char			if_interface[IFNAMSIZ];
	long			if_flags;
	u_long			if_qcuse;	/* Number of QC packets sent */
	OSI_DL_ADRS		es_multicast_addr;
	OSI_DL_ADRS		is_multicast_addr;
};	

/* Flags */
#define IND_IF_LISTEN_ES	0x02	/* Listen on all-ES */
#define IND_IF_LISTEN_IS	0x04	/* Listen on all-IS */
#define IND_IF_SEND_ES		0x08	/* Send on the ES address */
#define IND_IF_SEND_IS		0x10	/* Send on the IS address */
#define IND_IF_REDIRECT		0x20	/* Send(IS)/receive(ES) redirects */

#define	IND_ADD_IF_INFO		_IOW(o,40,struct ind_if_info)
#define	IND_DEL_IF_INFO		_IOW(o,41,struct ind_if_info)
#define IND_FLUSH_IF_INFO	_IO(o,42)

/* On call the parameter point to an integer num.
 * Return the if_info entry with index num. Returns EADDRNOTAVAIL if
 * num is larger than the number of available if_info entries.
 */
#define	IND_GET_IF_INFO		_IOWR(o,43,struct ind_if_info)

/* ------------------------------------------------------------------------- */

/* Configuration addresses */

struct osi_src_adrs {
	OSI_NSAP_ADRS	sa_nsap;
	u_long		sa_out;		/* Number of packets sent */
	u_long		sa_in;		/* Number of packets received */
	u_long		sa_flags;
};

#define OSI_SAF_NOT_SOURCE	0x01	/* Don't use the address as source
					   in datagrams */
#define OSI_SAF_NOT_LISTEN	0x02	/* Don't accept datagrams to this
					   NSAP */
#define OSI_SAF_NO_LOOPBACK	0x04	/* No loopback route */
#define OSI_SAF_NO_ESH		0x08	/* Don't advertise in ESH */

#define	IND_SET_MY_ADDR_DEFAULT _IOW(o,50,struct osi_src_adrs)
#define	IND_GET_MY_ADDR_DEFAULT _IOR(o,51,struct osi_src_adrs)
#define	IND_DEL_MY_ADDR_DEFAULT _IO(o,52)

#define IND_ADD_TO_SOURCE	_IOW(o,53,struct osi_src_adrs)
#define	IND_DEL_FROM_SOURCE	_IOW(o,54,struct osi_src_adrs)
#define	IND_FLUSH_SOURCE	_IO(o,55)

/* On call the parameter point to an integer num.
 * Return the optional source address with index num. Returns EADDRNOTAVAIL if
 * num is larger than the number of available optional source addresses.
 */
#define	IND_GET_SOURCE		_IOWR(o,56,struct osi_src_adrs)

#define	IND_SET_N_E_T		_IOW(o,57,struct osi_src_adrs)
#define	IND_GET_N_E_T		_IOR(o,58,struct osi_src_adrs)
#define	IND_DEL_N_E_T		_IO(o,59)

/* ------------------------------------------------------------------------- */

/* ES/IS operation */

struct ind_network_dst {
	char		nd_interface[IFNAMSIZ];
	OSI_DL_ADRS	nd_snpa;
};

#define	IND_BROADCAST_HELLO	_IO(o,60)
#define	IND_SEND_ESH_SINGLE	_IOW(o,61,struct ind_network_dst)
#define	IND_SEND_ISH_SINGLE	_IOW(o,62,struct ind_network_dst)


/* ------------------------------------------------------------------------- */

/* Routing tables */

typedef struct {
	char 		rt_interface[IFNAMSIZ];
	short	 	rt_flags;
	OSI_NSAP_ADRS	rt_nsap;	/* CLNS address		   */
	OSI_DL_ADRS	rt_snpa;	/* data link address	   */
	u_long		rt_use;		/* number of packets routed*/
	u_long		rt_in;		/* number of packets received */
	/* This is only used by the PREFIX routes */
	u_char		rt_prefixlen; 	/* in nibbles */
	/* These two only for OSI_RTF_PREFIX_EXTRACT */
	u_char		rt_extracttype;	/* in octets */	
	u_char		rt_extractparm;	/* in octets */	
} OSI_RTINFO;

#define	IND_ADD_ROUTE		_IOW(o,70,OSI_RTINFO)
#define	IND_DEL_ROUTE		_IOW(o,71,OSI_RTINFO)
#define IND_FLUSH_ROUTES	_IO(o,72)

#define	IND_ADD_INACTIVE	_IOW(o,73,OSI_RTINFO)
#define	IND_DEL_INACTIVE	_IOW(o,74,OSI_RTINFO)
#define IND_FLUSH_INACTIVE	_IO(o,75)

#define	IND_ADD_IS		_IOW(o,76,OSI_RTINFO)
#define	IND_DEL_IS		_IOW(o,77,OSI_RTINFO)
#define IND_FLUSH_IS		_IO(o,78)

/* On call the parameter point to an integer num.
 * Return the routing entry with index num. Returns EADDRNOTAVAIL if
 * num is larger than the number of available routing entries.
 */
#define	IND_GET_ROUTE		_IOWR(o,79,OSI_RTINFO)
#define	IND_GET_INACTIVE	_IOWR(o,80,OSI_RTINFO)
#define	IND_GET_IS		_IOWR(o,81,OSI_RTINFO)

/* ------------------------------------------------------------------------- */

/* Registration on the subnetwork. Currently only used for llc */

#define IND_REG_MAXLEN1		20	/* Octets */
#define IND_REG_MAXLEN2		20	/* Digits (i.e. nibbles) */
struct ind_net_register {
	char		ir_interface[IFNAMSIZ];
	unsigned	ir_len1;	/* In octets */
	u_char		ir_val1[IND_REG_MAXLEN1];
	unsigned	ir_len2;	/* In digits */
	u_char		ir_val2[(IND_REG_MAXLEN2+1)/2];
};

#define IND_REGISTER_ADD	_IOW(o, 90, struct ind_net_register)
#define IND_REGISTER_DEL	_IOW(o, 91, struct ind_net_register)
#define IND_REGISTER_FLUSH	_IO(o, 92)

/* On call the parameter point to an integer num.
 * Return the net_register entry with index num. Returns EADDRNOTAVAIL if
 * num is larger than the number of available net_registerentries.
 */
#define IND_REGISTER_GET	_IOWR(o, 93, struct ind_net_register)

/* ------------------------------------------------------------------------- */

#define IND_SET_LIFETIME	_IOW(o,97,int)
#define IND_SET_FLAGS		_IOW(o,98,int)

/* XXX Testing */
#define IND_KMEM_ALLOC		_IOW(o,99,long)


