/**************************************************************************
 FILE:
	ind.h
	
	@(#)ind.h 1.32 92/11/09 Copyr 1986 Sun Micro

 DESCRIPTION:
	Definitions for ISO CLNS network layer protocol. Per ISO 8473
	
 **************************************************************************/

#define GET_WORD(cp)	( (u_short)((cp)[0] << 8) | \
			  (u_short)((cp)[1]) )
			 
#define PUT_WORD(w, cp)	( (cp)[0] = (((w) >> 8) & 0xff), \
			  (cp)[1] = ((w) & 0xff) )

#define	IND_IS_END_SYS		0x1
#define	IND_IS_INT_SYS		0x2

#define OSI_IND_ADDR_LEN_FIELD_LEN	1 /* len. of adrs field's len. field*/


/* ------------------------------------------------------------------------- */
/* Protocol information */

#define IND_PTINACTIVE		(u_char)0x00
#define IND_PTINTERNET		(u_char)0x81
#define	IND_PTROUTING		(u_char)0x82


/* The inactive network layer header */
#define IND_HDR_ILEN	1	/* length of inactive header */


/* ES/IS */
#define	IND_ROUTE_ESH_ID	0x02
#define	IND_ROUTE_ISH_ID	0x04
#define	IND_ROUTE_RD_ID		0x06
#define MAX_ROUTING_PKT		254	/* max length of ES/IS packet */

/* The active network layer header */
#define IND_HDR_FPLEN	9	/* length of fixed part of header */
#define IND_HDR_SPLEN	6	/* length of segmentation part of header */


#define IND_FLG_SEGPERM		0x80	
#define IND_FLG_MORESEG		0x40
#define IND_FLG_ER_RPRT		0x20
		 
#define IS_SEGPERM(flag)	(((unsigned)(flag) & IND_FLG_SEGPERM) >> 7)
#define IS_MORESEG(flag)	(((unsigned)(flag) & IND_FLG_MORESEG) >> 6)
#define IS_ER_RPRT(flag)	(((unsigned)(flag) & IND_FLG_ER_RPRT) >> 5)

#define IND_VERSION		1

#define PKT_TYPE(flag)		((unsigned)(flag) & 0x1f)
#define IND_PKDT		0x1c	/* Data PDU */
#define IND_PKER		0x01	/* Error PDU */
/* ICNCFIX by -TH- for long term ping. */
#define IND_ECHORQ		0x1e	/* Echo request PDU */
#define IND_ECHORP		0x1f	/* Echo response PDU */
/* End of fix */

/* ------------------------------------------------------------------------- */
/* Default values */

#define IND_LIFEMAX		16	/* Default lifetime */
#define IND_REASS_TIMEOUT	16	/* Default reassembly timeout */

/* ------------------------------------------------------------------------- */
/* Reason codes */

typedef enum {
	IND_DR_UNSPEC 	= 0x00,	/* reason not specified */
	IND_DR_PROTO	= 0x01,	/* unspecfied protocol precedure error */
	IND_DR_CHKSUM 	= 0x02,	/* incorrect checksum */
	IND_DR_BUSY 	= 0x03,	/* discard due to congestion */
	IND_DR_SYNTAX 	= 0x04,	/* header syntax error */
	IND_DR_SEGREQ 	= 0x05,	/* segmentation is needed but not permitted */
	
	IND_DR_ADDRUNR 	= 0x80,	/* destination address unreachable */
	IND_DR_ADDRUNK 	= 0x81,	/* destination address unknown */
	
	IND_DR_SRCRUNSPC = 0x90,/* unspecifed source routing error */
	IND_DR_SRCRSYNTX = 0x91,/* syntax error in source routing field */
	IND_DR_SRCRUKN 	= 0x92,	/* unknown address in source routing field */
	IND_DR_SRCRPATH = 0x93,	/* path not acceptable */
	
	IND_DR_LIFETRAN = 0xa0,	/* lifetime expired while in transit */
	IND_DR_LIFEASSM = 0xa1,	/* lifetime expired during reassembly */
	
	IND_DR_OPTUNSPC = 0xb0,	/* unsupported option not specified */
	IND_DR_OPTPAD 	= 0xb1,	/* unsupported padding option */
	IND_DR_OPTSEC 	= 0xb2,	/* unsupported security option */
	IND_DR_OPTSRCR 	= 0xb3,	/* unsupported source routing option */
	IND_DR_OPTRECR 	= 0xb4,	/* unsupported route recording option */
	IND_DR_OPTQOS 	= 0xb5,	/* unsupported qos option */
	
	IND_DR_OK 	= 0xffff /* no error */
} IND_DISCR;

/* Internally used */
struct indp_arg_err {		/* error pdu data request arguments */
	IND_DISCR	reason;		/* error reason */
	unsigned	amount;		/* amount of header to send */
};

/* ------------------------------------------------------------------------- */
/* Options */

#define IND_OPT_PAD	0xcc	/* padding option */
#define IND_OPT_SRCR	0xc8	/* source routing option */
#define IND_OPT_RECR	0xcb	/* route recording option */
#define IND_OPT_QOS	0xc3	/* quality of service option */
#define IND_OPT_PRI	0xcd	/* priority option */
#define IND_OPT_SEC	0xc5	/* security option */
#define IND_OPT_ERR	0xc1	/* error reason options */

/* Options for ES-IS */
#define IND_OPT_ADDR_MASK	0xe1	/* Only in RD */
#define IND_OPT_AM_MAXLEN	20
#define IND_OPT_SNPA_MASK	0xe2	/* Only in RD with ADDR_MASK option */
#define IND_OPT_SM_MAXLEN	20
#define IND_OPT_ESCT		0xc6	/* Suggested Config Timer; only in ISH */ 
#define IND_OPT_ESCT_LEN	2


#define IND_OPT_RRSIZE	((MAX_NSAP_LEN + 1)*2)	/* route recording max size */
#define IND_OPT_RRLEN	(IND_OPT_RRSIZE + 4) 	/* size of RR parameter */
#define	IND_RR_ACTIVE0	0	/* partial RR/SR active */
#define	IND_RR_ACTIVE1	1	/* complete RR/SR active */
#define	IND_RR_TERM	0xff	/* RR terminated */

#define IND_SEC_BASIC	0x82	/* Basic security */
#define IND_SEC_UNCLASS	0x55	/* Unclassified */
#define IND_SEC_UFLAGS	1	/* Unclassified accreditation */
#define IND_OPT_SECLEN	6	/* length of security parameter */ 

#define IND_OPT_QOSLEN	3	/* length of QOS parameters */

/* Default values */
#define IND_OPT_DFLTQOS	0xc7	/* default QOS */
	
#define put1(cp, len, val) \
	{*(cp)++ = val; len++;}

#define copyopt(src, cp, len) \
	{bcopy((char *)src, (char *)cp, src[1] + 2); len += src[1] + 2;\
	 cp += src[1] + 2;}


/* ------------------------------------------------------------------------- */
/* Network layer headers */

struct indp_hdr_fp {		/* fixed part */
	u_char		proto;		/* protocol id (IND_PT*) */
	u_char		hlen;		/* header length */
	u_char		version;	/* protocol version */
	u_char		lifetime;	/* time to live */
	u_char		flags;		/* packet type and flags */
	u_char		slen[2];	/* segment length */
	u_char		chksum[2];	/* checksum */
};

struct indp_hdr_sp {		/* segmentation part */
	u_char		dui[2];		/* data unit idenifier */
	u_char		segoff[2];	/* segment offset */
	u_char		tlen[2];	/* total length */
};

struct indp_hdr_ap {		/* address part */
	u_char		alen;			/* address length */
	u_char		addr[MAX_NSAP_LEN];	/* address encoding */
};

/* ------------------------------------------------------------------------- */
/* Fragmentation stuctures */

struct indp_fragh {		/* fragment header (one per fragment) */
	struct indp_fragh *fnext, *fprev;/* to other pieces of this fragment */
	u_short		slen;		/* segment length (w/o header) */
	u_short		segoff;		/* segment offset */
	struct mbuf	*fragment;	/* pointer to mbuf chain that contains
					   TPDU */
};

struct indp_fragq {		/* fragment queue element header 
				   (one per NSDU) */
	struct indp_fragh *fnext, *fprev;/* to pieces of this fragment */
	struct indp_fragq *next, *prev;	/* to other fragments */
	u_short		dui;		/* data unit idenifier */
	u_short		ttl;		/* time to live */
	u_short		tlen;		/* total length */
	struct mbuf 	*mhead;		/* CLNP header */
	struct indp_hdr_ap	sa, da;		/* Source and dest */	
};

/* ------------------------------------------------------------------------- */
/* Option structures */

struct indp_hdr_rr {		/* route recording parameter */
	u_char		code;		/* parameter code */
	u_char		size;		/* parameter size */
	u_char		status;		/* route recording active */
	u_char		offset;		/* offset of end of rr data */
	u_char		data[1]; 	/* rr data */
};

struct indp_hdr_qos {		/* quality-of-service parameter */
	u_char		code;		/* parameter code */
	u_char		size;		/* parameter size */
	u_char		qos;		/* data */
};
	
struct indp_hdr_err {		/* error reason parameter */
	u_char		code;		/* parameter code */
	u_char		size;		/* parameter size */
	u_char		reason[2];	/* data */
};
	
struct indp_hdr_sec {		/* security parameter */
	u_char		code;		/* parameter code */
	u_char		size;		/* parameter size */
	u_char		format;		/* 8473 format code */
	/* US-GOSIP basic security */
	u_char		option;		/* security option */
	u_char		class;		/* classification level */
	u_char		flags;		/* accreditation flags */
};

/* ------------------------------------------------------------------------- */
/*
 * Network layer hash table bucket entry
 */
struct	ind_client {
	u_char		nsel;		/* client selector */
	struct protosw	*client;	/* client protocol entry */
	int		reference_cnt;	/* number of times listener
					 * requested this address
					 */
	int		flags;		/* See below */
	struct ind_client *next;	/* next client in hash bucket */
};

/* Flags (0 in the normal case) */
#define	CLF_RAW		0x01		/* Receive the header + data */
#define CLF_ER		0x02		/* Receive ER PDUs destined to 
					 * the NSEL */

#define CHASH_NVAL	256
#define CHASH_MAXVAL	(CHASH_NVAL-1)
#define	CHASH_SHIFT	3
#define CHASH_SIZE	((CHASH_MAXVAL >> CHASH_SHIFT)+1)
#define CHASH(sel)	((u_char)(sel) >> CHASH_SHIFT)
extern struct	ind_client  *chash[CHASH_SIZE];	/* hash table for clients */


/* ------------------------------------------------------------------------- */
/*
 * Control output arg commands and structures
 */
#define INDCNTL_SETCLIENT	1
#define INDCNTL_CLRCLIENT	2

struct ind_newclient {
	struct ind_client	*client_handle;
};

/* ------------------------------------------------------------------------- */

/*
 * Network layer interface.
 */
struct nexthop {
	OSI_DL_ADRS	snpa;
	struct ifnet	*ifp;
};


/* ------------------------------------------------------------------------- */
/*
 * Debugging aids
 */
#ifdef INDDEBUG

#define	IND_DBG_ROUTE	0x0001
#define	IND_DBG_INPUT	0x0002
#define	IND_DBG_ADDR	0x0004
#define	IND_DBG_INACT	0x0008
#define	IND_DBG_FRAG	0x0010
#define	IND_DBG_OPT	0x0020
#define	IND_DBG_OUTPUT	0x0040
#define	IND_DBG_USER	0x0080
#define	IND_DBG_HASH	0x0100

#endif

/* ------------------------------------------------------------------------- */

#ifdef KERNEL
/* Input queue from subnet */
extern struct    ifqueue indintrq;

struct ifnet	osi_clns_dev;
long		router_status;

struct osi_src_adrs 	*ind_get_my_addr(/* */);
struct osi_src_adrs 	*ind_get_n_e_t(/* */);
struct osi_src_adrs 	*ind_get_source(/*int index*/);

struct ifnet 	*ind_get_if_info_es(/* int index, OSI_DL_ADRS *es_multicast*/);
struct ifnet 	*ind_get_if_info_is(/* int index, OSI_DL_ADRS *is_multicast*/);
OSI_DL_ADRS 	*ind_get_qc_addr(/* struct ifnet *ifp */);
void 		ind_if_info_incr_use(/* struct ifnet *ifp */);
void 		ind_find_qcs(/* struct nexthop *nexthops, int *numhops */);
void 		ind_find_iss(/* struct nexthop *nexthops, int *numhops */);

struct ifnet	*ind_find_inactive_nsap();
struct ifnet	*ind_route();

#endif KERNEL

/* ------------------------------------------------------------------------- */



