/*
 * Copyright (c) 1980, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	@(#)osi_route.h 1.12 91/01/11 SMI; modified from UCB route.h 7.1 6/86
 *
 * Modified for OSI by Sun Microsystems, Inc., August 1988
 */

/*
 * This is necessary so that "struct sockaddr" is defined for the
 * OSI_DL_ADRS typedef when osi_route.h is included in ses_stream.c.
 */
#ifndef	_SOCKET_
#include <sys/socket.h>
#endif

/*
 * Available OSI "families."
 *
 *	OSI_AF_MIN <= family < OSI_AF_MAX
 *
 * The following is done for backward compatibility:
 *
 *	-- #defines for OSI_FAMILY values are done in osi_addr.h
 *	   to provide the old "AF_*" syntax (e.g. "AF_NBS," "AF_ERROR").
 *
 *	-- OSI_AF_UNSPEC and OSI_AF_ERROR are set to zero so that
 *	   their #define equivalents are the same value as "AF_ERROR"
 *	   and "AF_UNSPEC" in socket.h.
 */
typedef enum {
	OSI_AF_UNSPEC,		/* Nothing specified (see osi_clr_sap()). */
	OSI_AF_ERROR,		/* Error condition.			  */
	OSI_AF_GENERIC,		/* Used by non-network layers as family.  */
	OSI_AF_USER_DEFINED,	/* user_defined	family			  */
	OSI_AF_NBS,		/* nbs		family			  */
	OSI_AF_OSINET,		/* osinet	family			  */
	OSI_AF_CONS_1980,	/* cons-1980	family			  */
	OSI_AF_ENE,		/* ene		family			  */
	OSI_AF_TCP,		/* RFC 1006 TCP address in an NSAP	  */
	OSI_AF_MAX		/* MUST BE *LAST* */
} OSI_FAMILY;

#define OSI_AF_MIN	((unsigned) OSI_AF_UNSPEC)

/* ------------------------------------------------------------------------- */
/*
 * Kernel resident routing tables.
 * 
 */

/*
 * Must leave some room for nesting an OSI_NSAP_ADRS
 * in a higher level structure, where the higher
 * level structure may all be contained in
 * an mbuf (e.g. osi_rtentry, src_addr_list).
 * osi_rtentry is currently the largest such
 * structure.
 *
 * 40 digits or 20 octets is the maximum NSAP size
 * (see ISO 8348/DAD2, section 8.4).
 *
 * This should leave plenty of room for other fields
 * in the osi_rtentry structure (to fit in an mbuf).
 */
#define MAX_NSAP_LEN		20
#define MIN_NSAP_LEN		2 	/* conservative; AFI + NSEL */

typedef struct {
	OSI_FAMILY	osi_nsap_family;
	u_char		osi_nsap_len;
	u_char		osi_nsap_adrs[MAX_NSAP_LEN];
} OSI_NSAP_ADRS;

typedef struct sockaddr		OSI_DL_ADRS;
#define OSI_DL_DATA_SIZE	SOCKADDR_DATA_SIZE

/* ------------------------------------------------------------------------- */
/*
 * OSI_RTF_UP:		used primarily to indicate i/f down when route rm'd
 * OSI_RTF_HOST:	used primarily as indicator to install host entry
 */
#define	OSI_RTF_UP		0x0001	/* route useable		*/
#define	OSI_RTF_HOST		0x0002	/* host entry			*/
#define OSI_RTF_PREFIX		0x0004	/* prefix with a length in nibbles */
#define OSI_RTF_PREFIX_LEN	0x0008	/* prefix + specified length */
#define OSI_RTF_PREFIX_EXTRACT	0x0010	/* prefix_len + extract SNPA */
#define OSI_RTF_IDP_PREFIX	0x0020	/* prefix that ends in the IDP */
#define OSI_RTF_IDP_EXTRACT	0x0040	/* idp_prefix + extract SNPA */

/* All but "UP" */
#define OSI_RTF_ROUTE_MASK	\
(OSI_RTF_HOST|OSI_RTF_PREFIX|OSI_RTF_PREFIX_LEN|OSI_RTF_PREFIX_EXTRACT|\
OSI_RTF_IDP_PREFIX|OSI_RTF_IDP_EXTRACT)

/* These flags for osirstat display only */
#define OSI_RTF_NET		0x0100	/* Next hop is a NET */
#define OSI_RTF_U1		0x0200
#define OSI_RTF_U2		0x0400
#define OSI_RTF_U3		0x0800
#define OSI_RTF_STATIC		0x1000	/* static route */
#define OSI_RTF_ESH		0x2000	/* route caused by a multicast hello */
#define OSI_RTF_DIRESH		0x4000	/* from directed hello */
#define OSI_RTF_REDIR		0x8000	/* from a redirect */

#define OSI_RTF_USER_SETTABLE	\
(OSI_RTF_UP|OSI_RTF_NET|OSI_RTF_U1|OSI_RTF_U2|OSI_RTF_U3|\
OSI_RTF_STATIC|OSI_RTF_ESH|OSI_RTF_DIRESH|OSI_RTF_REDIR)

/* ------------------------------------------------------------------------- */
#ifdef KERNEL

/*
 * We distinguish between routes to hosts and routes to networks,
 * preferring the former if available.  For each route we infer
 * the interface to use from the immediate destination address supplied
 * when the route was entered.  Routes that forward packets through
 * gateways are marked so that the output routines know to address the
 * gateway rather than the ultimate destination.
 */
typedef struct {
	u_long		 osi_rt_hash;
	OSI_NSAP_ADRS	 osi_rt_nsap;	/* CLNS address		   */
	OSI_DL_ADRS	 osi_rt_snpa;	/* data link address	   */
	short		 osi_rt_flags;
	short		 osi_rt_refcnt;	/*see osi_rtalloc() comment*/
	u_long		 osi_rt_use;	/* number of packets routed*/
	struct ifnet	*osi_rt_ifp;	/* interface		   */
	/* This is only used by the PREFIX routes */
	u_char		 osi_rt_prefixlen; /* in nibbles */
	/* These two only for OSI_RTF_PREFIX_EXTRACT */
	u_char		 osi_rt_extracttype;	/* see below */	
	u_char		 osi_rt_extractparm;	/* parameter */	
} OSI_RTENTRY;

/*
 * A route consists of a destination address and a reference
 * to a routing entry.
 */
typedef struct {
	OSI_RTENTRY	*osi_ro_rt; /* immediate destination info.	*/
	OSI_NSAP_ADRS	osi_ro_dst; /* ultimate destination requested	*/
} OSI_ROUTE;

struct osi_rt_handle {
	struct mbuf 	**host_table, **gateway_table, **prefix_table; 
	int		host_size, gateway_size; /* Must be OSI_RT_SIZE */
	int		prefix_size;		/* Always 2*MAX_NSAP_LEN */
};
#endif KERNEL

/* Address extraction types */
#define OSI_ET_NONE	0

/* 
 * Extract a fixed length SNPA from the NSAP.
 * The extract parameter is the offset in the NSAP (1..20) where the
 * extraction starts.
 */
#define	OSI_ET_MAC	1	/* 6 byte mac address. */
#define	OSI_ET_LLC	2	/* 7 byte mac+lsel address. */
#define	OSI_ET_IP	3	/* 4 byte IP address */

/*
 * Extract a variable length SNPA from the NSAP.
 * The extract parameter the length of the IDI. The IDI is assumed to
 * start at byte 2.
 */
#define	OSI_ET_X25_0	10	/* Skip leading 0 digits in the IDI */	
#define	OSI_ET_X25_1	11	/* Skip leading 1 digits in the IDI */	


/* ------------------------------------------------------------------------- */
/*
 * Route hashing
 */

#ifdef	GATEWAY
#define	OSI_RTHASHSIZ	64
#else
#define	OSI_RTHASHSIZ	8
#endif

#if	(OSI_RTHASHSIZ & (OSI_RTHASHSIZ - 1)) == 0
#define OSI_RTHASHMOD(h)	((h) & (OSI_RTHASHSIZ - 1))
#else
#define OSI_RTHASHMOD(h)	((h) % OSI_RTHASHSIZ)
#endif

/*
 * NOTE: we assume that the nsel is the last byte of the address and
 *	 we don't route on it.
 *
 * ASSUMPTION: nsel is last byte
 *
 * Returns:
 *	addresses not equal:	zero
 *	addresses equal:	non-zero
 */
#define OSI_HOSTMATCH(addr1, addr2)					\
	((addr1)->osi_nsap_family == (addr2)->osi_nsap_family	&&	\
	 (addr1)->osi_nsap_len    == (addr2)->osi_nsap_len &&		\
	 (bcmp((char *)(addr1)->osi_nsap_adrs,				\
	       (char *)(addr2)->osi_nsap_adrs,				\
	       (int)(addr1)->osi_nsap_len - UDNL) == 0))

/*
 * Just like OSI_HOSTMATCH(), but includes the NSEL.
 * NOTE: this is like a bcmp() of the two complete NSAPs, except
 *	 that it only checks the relevant bytes of the NSAP adrs.s
 *	 themselves -- this prevents an NSAP which hasn't been bzero()'d
 *	 before initialization from failing to match with another
 *	 identical NSAP.
 */
#define OSI_NSAP_CMP(addr1, addr2)					\
	((addr1)->osi_nsap_family == (addr2)->osi_nsap_family	&&	\
	 (addr1)->osi_nsap_len    == (addr2)->osi_nsap_len &&		\
	 (bcmp((char *)(addr1)->osi_nsap_adrs,				\
	       (char *)(addr2)->osi_nsap_adrs,				\
	       (int)(addr1)->osi_nsap_len) == 0))

/* ------------------------------------------------------------------------- */
#ifdef KERNEL
#define	OSI_RTFREE(osi_rt) \
	if ((osi_rt)->osi_rt_refcnt == 1) \
		osi_rtfree(osi_rt); \
	else \
		(osi_rt)->osi_rt_refcnt--;

struct osi_afhash {
	u_int   osi_afh_hosthash;
	u_int   osi_afh_nethash;
};
#endif
/* ------------------------------------------------------------------------- */


